/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.datamodel;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.7 $
 */
public class AlignmentAnnotation
{
    /** DOCUMENT ME!! */
    public String label;

    /** DOCUMENT ME!! */
    public String description;

    /** DOCUMENT ME!! */
    public Annotation[] annotations;

    /** DOCUMENT ME!! */
    public boolean isGraph = false;

    /** DOCUMENT ME!! */
    public float graphMin;

    /** DOCUMENT ME!! */
    public float graphMax;

    /** DOCUMENT ME!! */
    public int windowLength;

    // Graphical hints and tips

    /** DOCUMENT ME!! */
    public boolean editable = false;

    /** DOCUMENT ME!! */
    public boolean hasIcons; //

    /** DOCUMENT ME!! */
    public boolean hasText;

    /** DOCUMENT ME!! */
    public boolean visible = true;

    /** DOCUMENT ME!! */
    public int height = 0;

    /**
     * Creates a new AlignmentAnnotation object.
     *
     * @param label DOCUMENT ME!
     * @param description DOCUMENT ME!
     * @param annotations DOCUMENT ME!
     */
    public AlignmentAnnotation(String label, String description,
        Annotation[] annotations)
    {
        // always editable?
        editable = true;
        this.label = label;
        this.description = description;
        this.annotations = annotations;

        for (int i = 0; i < annotations.length; i++)
        {
            if ((annotations[i] != null) &&
                    ((annotations[i].secondaryStructure == 'H') ||
                    (annotations[i].secondaryStructure == 'E')))
            {
                hasIcons = true;
            }

            if ((annotations[i] != null) &&
                    (annotations[i].displayCharacter.length() > 0))
            {
                hasText = true;
            }
        }
    }

    /**
     * Creates a new AlignmentAnnotation object.
     *
     * @param label DOCUMENT ME!
     * @param description DOCUMENT ME!
     * @param annotations DOCUMENT ME!
     * @param min DOCUMENT ME!
     * @param max DOCUMENT ME!
     * @param winLength DOCUMENT ME!
     */
    public AlignmentAnnotation(String label, String description,
        Annotation[] annotations, float min, float max, int winLength)
    {
        // graphs are not editable
        this.label = label;
        this.description = description;
        this.annotations = annotations;
        isGraph = true;

        if (min == max)
        {
            for (int i = 0; i < annotations.length; i++)
            {
                if (annotations[i] == null)
                {
                    continue;
                }

                if (annotations[i].value > max)
                {
                    max = annotations[i].value;
                }

                if (annotations[i].value < min)
                {
                    min = annotations[i].value;
                }
            }
        }

        graphMin = min;
        graphMax = max;
        windowLength = winLength;

        for (int i = 0; i < annotations.length; i++)
        {
            if ((annotations[i] != null) &&
                    ((annotations[i].secondaryStructure == 'H') ||
                    (annotations[i].secondaryStructure == 'E')))
            {
                hasIcons = true;
            }

            if ((annotations[i] != null) &&
                    (annotations[i].displayCharacter.length() > 0))
            {
                hasText = true;
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();

        for (int i = 0; i < annotations.length; i++)
        {
            if (annotations[i] != null)
            {
                if (isGraph)
                {
                    buffer.append(annotations[i].value);
                }
                else if (hasIcons)
                {
                    buffer.append(annotations[i].secondaryStructure);
                }
                else
                {
                    buffer.append(annotations[i].displayCharacter);
                }
            }

            buffer.append(", ");
        }

        if (label.equals("Consensus"))
        {
            buffer.append("\n");

            for (int i = 0; i < annotations.length; i++)
            {
                if (annotations[i] != null)
                {
                    buffer.append(annotations[i].description);
                }

                buffer.append(", ");
            }
        }

        return buffer.toString();
    }
}
