/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import java.util.*;


/**
 * NOTE: Columns are zero based.
 */
public class ColumnSelection
{
    Vector selected = new Vector();

    /**
     * DOCUMENT ME!
     *
     * @param col DOCUMENT ME!
     */
    public void addElement(int col)
    {
        if (!selected.contains(new Integer(col)))
        {
            selected.addElement(new Integer(col));
        }
    }

    /**
     * DOCUMENT ME!
     */
    public void clear()
    {
        selected.removeAllElements();
    }

    /**
     * DOCUMENT ME!
     *
     * @param col DOCUMENT ME!
     */
    public void removeElement(int col)
    {
        Integer colInt = new Integer(col);

        if (selected.contains(colInt))
        {
            selected.removeElement(colInt);
        }
    }

    public void removeElements(int start, int end)
    {
      Integer colInt;
      for(int i=start; i<end; i++)
      {
        colInt = new Integer(i);
        if (selected.contains(colInt))
        {
            selected.removeElement(colInt);
        }
      }
    }

    /**
     * DOCUMENT ME!
     *
     * @param col DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean contains(int col)
    {
        return selected.contains(new Integer(col));
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int columnAt(int i)
    {
        return ((Integer) selected.elementAt(i)).intValue();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int size()
    {
        return selected.size();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getMax()
    {
        int max = -1;

        for (int i = 0; i < selected.size(); i++)
        {
            if (columnAt(i) > max)
            {
                max = columnAt(i);
            }
        }

        return max;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getMin()
    {
        int min = 1000000000;

        for (int i = 0; i < selected.size(); i++)
        {
            if (columnAt(i) < min)
            {
                min = columnAt(i);
            }
        }

        return min;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Vector asVector()
    {
        return selected;
    }

    /**
     * DOCUMENT ME!
     *
     * @param start DOCUMENT ME!
     * @param change DOCUMENT ME!
     */
    public void compensateForEdit(int start, int change)
    {
        for (int i = 0; i < size(); i++)
        {
            int temp = columnAt(i);

            if (temp >= start)
            {
                selected.setElementAt(new Integer(temp - change), i);
            }
        }
    }
}
