/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.analysis.*;

import jalview.datamodel.*;

import jalview.jbgui.*;

import java.awt.event.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.8 $
 */
public class PairwiseAlignPanel extends GPairwiseAlignPanel
{
    Vector sequences = new Vector();
    AlignViewport av;

    /**
     * Creates a new PairwiseAlignPanel object.
     *
     * @param av DOCUMENT ME!
     */
    public PairwiseAlignPanel(AlignViewport av)
    {
        super();
        this.av = av;

        Vector selsubset = new Vector();

        for (int i = 0, j = av.getSelectionGroup().getSize(); i < j; i++)
        {
            if (av.getAlignment().getSequences().contains(av.getSelectionGroup()
                                                                .getSequenceAt(i)))
            {
                selsubset.add(av.getSelectionGroup().getSequenceAt(i));
            }
        }

        float[][] scores = new float[selsubset.size()][selsubset.size()];
        double totscore = 0;
        int count = selsubset.size();

        int acount = 0;

        for (int i = 1; i < count; i++)
        {
            for (int j = 0; j < i; j++)
            {
                acount++;

                AlignSeq as = new AlignSeq((SequenceI) selsubset.elementAt(i),
                        (SequenceI) selsubset.elementAt(j), "pep");
                as.calcScoreMatrix();
                as.traceAlignment();
                as.printAlignment();
                scores[i][j] = (float) as.getMaxScore() / (float) as.getASeq1().length;
                totscore = totscore + scores[i][j];

                textarea.append(as.getOutput());
                sequences.add(new Sequence(as.getS1().getName(), as.getAStr1()));
                sequences.add(new Sequence(as.getS2().getName(), as.getAStr2()));
            }
        }

        if (count > 2)
        {
            System.out.println(
                "Pairwise alignment scaled similarity score matrix\n");

            for (int i = 0; i < count; i++)
            {
                jalview.util.Format.print(System.out, "%s \n",
                    ("" + i) + " " +
                    ((SequenceI) selsubset.elementAt(i)).getName());
            }

            System.out.println("\n");

            for (int i = 0; i < count; i++)
            {
                for (int j = 0; j < i; j++)
                {
                    jalview.util.Format.print(System.out, "%7.3f",
                        scores[i][j] / totscore);
                }
            }

            System.out.println("\n");
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void viewInEditorButton_actionPerformed(ActionEvent e)
    {
        Sequence[] seq = new Sequence[sequences.size()];

        for (int i = 0; i < sequences.size(); i++)
        {
            seq[i] = (Sequence) sequences.elementAt(i);
        }

        AlignFrame af = new AlignFrame(new Alignment(seq));
        Desktop.addInternalFrame(af, "Pairwise Aligned Sequences",
            AlignFrame.NEW_WINDOW_WIDTH, AlignFrame.NEW_WINDOW_HEIGHT);
    }
}
