/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import MCview.*;

import jalview.analysis.*;

import jalview.datamodel.*;

import jalview.io.*;

import jalview.schemes.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.35 $
 */
public class PopupMenu extends JPopupMenu
{
    JMenu groupMenu = new JMenu();
    JMenuItem groupName = new JMenuItem();
    protected JRadioButtonMenuItem clustalColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem zappoColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem taylorColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem hydrophobicityColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem helixColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem strandColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem turnColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem buriedColour = new JRadioButtonMenuItem();
    protected JCheckBoxMenuItem abovePIDColour = new JCheckBoxMenuItem();
    protected JRadioButtonMenuItem userDefinedColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem PIDColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem BLOSUM62Colour = new JRadioButtonMenuItem();
    JRadioButtonMenuItem noColourmenuItem = new JRadioButtonMenuItem();
    protected JCheckBoxMenuItem conservationMenuItem = new JCheckBoxMenuItem();
    AlignmentPanel ap;
    JMenu sequenceMenu = new JMenu();
    JMenuItem sequenceName = new JMenuItem();
    Sequence sequence;
    JMenuItem unGroupMenuItem = new JMenuItem();
    JMenuItem pdbMenuItem = new JMenuItem();
    JMenuItem outline = new JMenuItem();
    JRadioButtonMenuItem nucleotideMenuItem = new JRadioButtonMenuItem();
    JMenu colourMenu = new JMenu();
    JCheckBoxMenuItem showBoxes = new JCheckBoxMenuItem();
    JCheckBoxMenuItem showText = new JCheckBoxMenuItem();
    JCheckBoxMenuItem showColourText = new JCheckBoxMenuItem();

    /**
     * Creates a new PopupMenu object.
     *
     * @param ap DOCUMENT ME!
     * @param seq DOCUMENT ME!
     */
    public PopupMenu(AlignmentPanel ap, Sequence seq)
    {
        ///////////////////////////////////////////////////////////
        // If this is activated from the sequence panel, the user may want to
        // edit or annotate a particular residue. Therefore display the residue menu
        //
        // If from the IDPanel, we must display the sequence menu
        //////////////////////////////////////////////////////////
        this.ap = ap;
        sequence = seq;

        ButtonGroup colours = new ButtonGroup();
        colours.add(noColourmenuItem);
        colours.add(clustalColour);
        colours.add(zappoColour);
        colours.add(taylorColour);
        colours.add(hydrophobicityColour);
        colours.add(helixColour);
        colours.add(strandColour);
        colours.add(turnColour);
        colours.add(buriedColour);
        colours.add(abovePIDColour);
        colours.add(userDefinedColour);
        colours.add(PIDColour);
        colours.add(BLOSUM62Colour);

        try
        {
            jbInit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        if ((seq == null) || (seq.getPDBId() == null))
        {
            pdbMenuItem.setVisible(false);
        }

        SequenceGroup sg = ap.av.getSelectionGroup();

        if (sg != null)
        {
            groupName.setText(sg.getName());

            if (sg.cs instanceof ZappoColourScheme)
            {
                zappoColour.setSelected(true);
            }
            else if (sg.cs instanceof TaylorColourScheme)
            {
                taylorColour.setSelected(true);
            }
            else if (sg.cs instanceof PIDColourScheme)
            {
                PIDColour.setSelected(true);
            }
            else if (sg.cs instanceof Blosum62ColourScheme)
            {
                BLOSUM62Colour.setSelected(true);
            }
            else if (sg.cs instanceof UserColourScheme)
            {
                userDefinedColour.setSelected(true);
            }
            else if (sg.cs instanceof HydrophobicColourScheme)
            {
                hydrophobicityColour.setSelected(true);
            }
            else if (sg.cs instanceof HelixColourScheme)
            {
                helixColour.setSelected(true);
            }
            else if (sg.cs instanceof StrandColourScheme)
            {
                strandColour.setSelected(true);
            }
            else if (sg.cs instanceof TurnColourScheme)
            {
                turnColour.setSelected(true);
            }
            else if (sg.cs instanceof BuriedColourScheme)
            {
                buriedColour.setSelected(true);
            }
            else if (sg.cs instanceof ClustalxColourScheme)
            {
                clustalColour.setSelected(true);
            }
            else
            {
                noColourmenuItem.setSelected(true);
            }

            if (sg.cs!=null && sg.cs.conservationApplied())
            {
                conservationMenuItem.setSelected(true);
            }

            showText.setSelected(sg.getDisplayText());
            showColourText.setSelected(sg.getColourText());
            showBoxes.setSelected(sg.getDisplayBoxes());
        }

        if (!ap.av.alignment.getGroups().contains(sg))
        {
            unGroupMenuItem.setVisible(false);
        }
        else
        {
            groupMenu.insertSeparator(3);
        }

        if (seq == null)
        {
            sequenceMenu.setVisible(false);
        }
        else
        {
          java.util.Vector links = Preferences.sequenceURLLinks;

          JMenu linkMenu = new JMenu("Link");
          JMenuItem item;
          for(int i=0; i<links.size(); i++)
          {
            String link = links.elementAt(i).toString();

            item = new JMenuItem(link.substring(0, link.indexOf("|")));

            final String url = link.substring(link.indexOf("|")+1, link.indexOf("$SEQUENCE_ID$"))
                + sequence.getName() +
                link.substring(link.indexOf("$SEQUENCE_ID$") + 13);


            item.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showLink(url);
                }
            });

            linkMenu.add(item);
          }
          add(linkMenu);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    private void jbInit() throws Exception
    {
        groupMenu.setText("Group");
        groupMenu.setText("Define");
        groupName.setText("Name");
        groupName.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    groupName_actionPerformed(e);
                }
            });
        sequenceMenu.setText("Sequence");
        sequenceName.setText("Edit name");
        sequenceName.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    sequenceName_actionPerformed(e);
                }
            });
        PIDColour.setFocusPainted(false);
        unGroupMenuItem.setText("Remove Group");
        unGroupMenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    unGroupMenuItem_actionPerformed(e);
                }
            });
        pdbMenuItem.setText("View PDB structure");
        pdbMenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    pdbMenuItem_actionPerformed(e);
                }
            });
        outline.setText("Border colour");
        outline.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    outline_actionPerformed(e);
                }
            });
        nucleotideMenuItem.setText("Nucleotide");
        nucleotideMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    nucleotideMenuItem_actionPerformed(e);
                }
            });
        colourMenu.setText("Group Colour");
        showBoxes.setText("Boxes");
        showBoxes.setState(true);
        showBoxes.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showBoxes_actionPerformed(e);
                }
            });
        showText.setText("Text");
        showText.setState(true);
        showText.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showText_actionPerformed(e);
                }
            });
        showColourText.setText("Colour Text");
        showColourText.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showColourText_actionPerformed(e);
                }
            });
        add(groupMenu);
        add(sequenceMenu);
        groupMenu.add(groupName);
        groupMenu.addSeparator();
        groupMenu.add(unGroupMenuItem);
        groupMenu.add(colourMenu);
        groupMenu.addSeparator();
        groupMenu.add(showBoxes);
        groupMenu.add(showText);
        groupMenu.add(showColourText);
        groupMenu.addSeparator();
        groupMenu.add(outline);
        sequenceMenu.add(sequenceName);
        sequenceMenu.add(pdbMenuItem);
        colourMenu.add(noColourmenuItem);
        colourMenu.add(clustalColour);
        colourMenu.add(BLOSUM62Colour);
        colourMenu.add(PIDColour);
        colourMenu.add(zappoColour);
        colourMenu.add(taylorColour);
        colourMenu.add(hydrophobicityColour);
        colourMenu.add(helixColour);
        colourMenu.add(strandColour);
        colourMenu.add(turnColour);
        colourMenu.add(buriedColour);
        colourMenu.add(nucleotideMenuItem);
        colourMenu.add(userDefinedColour);

        if(jalview.gui.UserDefinedColours.getUserColourSchemes()!=null)
        {
          java.util.Enumeration userColours = jalview.gui.UserDefinedColours.
              getUserColourSchemes().keys();

          while (userColours.hasMoreElements())
          {
            JMenuItem item = new JMenuItem(userColours.
                nextElement().toString());
            item.addActionListener(new ActionListener()
            {
              public void actionPerformed(ActionEvent evt)
              {
                userDefinedColour_actionPerformed(evt);
              }
            });
            colourMenu.add(item);
          }
        }


        colourMenu.addSeparator();
        colourMenu.add(abovePIDColour);
        colourMenu.add(conservationMenuItem);
        noColourmenuItem.setText("None");
        noColourmenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    noColourmenuItem_actionPerformed(e);
                }
            });

        clustalColour.setText("Clustalx colours");
        clustalColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    clustalColour_actionPerformed(e);
                }
            });
        zappoColour.setText("Zappo");
        zappoColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    zappoColour_actionPerformed(e);
                }
            });
        taylorColour.setText("Taylor");
        taylorColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    taylorColour_actionPerformed(e);
                }
            });
        hydrophobicityColour.setText("Hydrophobicity");
        hydrophobicityColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    hydrophobicityColour_actionPerformed(e);
                }
            });
        helixColour.setText("Helix propensity");
        helixColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    helixColour_actionPerformed(e);
                }
            });
        strandColour.setText("Strand propensity");
        strandColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    strandColour_actionPerformed(e);
                }
            });
        turnColour.setText("Turn propensity");
        turnColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    turnColour_actionPerformed(e);
                }
            });
        buriedColour.setText("Buried Index");
        buriedColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    buriedColour_actionPerformed(e);
                }
            });
        abovePIDColour.setText("Above % Identity");
        abovePIDColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    abovePIDColour_actionPerformed(e);
                }
            });
        userDefinedColour.setText("User Defined...");
        userDefinedColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    userDefinedColour_actionPerformed(e);
                }
            });
        PIDColour.setText("Percentage Identity");
        PIDColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    PIDColour_actionPerformed(e);
                }
            });
        BLOSUM62Colour.setText("BLOSUM62");
        BLOSUM62Colour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    BLOSUM62Colour_actionPerformed(e);
                }
            });
        conservationMenuItem.setText("Conservation");
        conservationMenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    conservationMenuItem_actionPerformed(e);
                }
            });
    }

    /**
     * DOCUMENT ME!
     */
    void refresh()
    {
        SequenceGroup sg = getGroup();
        if(sg!=null)
        {
          SuperGroup superG = ap.av.alignment.getSuperGroup(sg);

          if (superG != null)
          {
            superG.setSuperGroupProperties(sg);
          }
        }
        ap.seqPanel.repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void clustalColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        sg.cs = new ClustalxColourScheme(sg.sequences,
                ap.av.alignment.getWidth());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void zappoColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new ZappoColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void taylorColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new TaylorColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void hydrophobicityColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new HydrophobicColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void helixColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new HelixColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void strandColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new StrandColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void turnColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new TurnColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void buriedColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new BuriedColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void nucleotideMenuItem_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new NucleotideColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void abovePIDColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        if(sg.cs==null)
          return;

        if (abovePIDColour.isSelected())
        {
            sg.cs.setConsensus(AAFrequency.calculate(sg.sequences, 0,
                    ap.av.alignment.getWidth()));

            int threshold = SliderPanel.setPIDSliderSource(ap, sg.cs,
                    getGroup().getName());

           sg.cs.setThreshold(threshold, ap.av.getIgnoreGapsConsensus());

            SliderPanel.showPIDSlider();
        }
        else // remove PIDColouring
        {
            sg.cs.setThreshold(0, ap.av.getIgnoreGapsConsensus());
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void userDefinedColour_actionPerformed(ActionEvent e)
    {
       SequenceGroup sg = getGroup();

       if (e.getActionCommand().equals("User Defined..."))
         new UserDefinedColours(ap, sg);
       else
       {
         UserColourScheme udc = (UserColourScheme) UserDefinedColours.
             getUserColourSchemes().get(e.getActionCommand());

         sg.cs = udc;
       }
     }
    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void PIDColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        sg.cs = new PIDColourScheme();
        sg.cs.setConsensus(AAFrequency.calculate(sg.sequences, 0,
                ap.av.alignment.getWidth()));
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void BLOSUM62Colour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();

        sg.cs = new Blosum62ColourScheme();

        sg.cs.setConsensus(AAFrequency.calculate(sg.sequences, 0,
                ap.av.alignment.getWidth()));

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void noColourmenuItem_actionPerformed(ActionEvent e)
    {
        getGroup().cs = null;
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void conservationMenuItem_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        if(sg.cs==null)
          return;

        if (conservationMenuItem.isSelected())
        {
            Conservation c = new Conservation("Group",
                    ResidueProperties.propHash, 3, sg.sequences, 0,
                    ap.av.alignment.getWidth());

            c.calculate();
            c.verdict(false, ap.av.ConsPercGaps);

            sg.cs.setConservation(c);

            SliderPanel.setConservationSlider(ap, sg.cs, sg.getName());
            SliderPanel.showConservationSlider();
        }
        else // remove ConservationColouring
        {
            sg.cs.setConservation(null);
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void groupName_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        String reply = JOptionPane.showInternalInputDialog(Desktop.desktop,
                "Enter new group name", "Edit group name",
                JOptionPane.QUESTION_MESSAGE);

        if (reply == null)
        {
            return;
        }

        sg.setName(reply);
        groupName.setText(reply);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void analyze_actionPerformed(ActionEvent e)
    {
        CutAndPasteTransfer cap = new CutAndPasteTransfer();
        JInternalFrame frame = new JInternalFrame();
        frame.setContentPane(cap);
        Desktop.addInternalFrame(frame, "Analyze this - ", 400, 300);

        SequenceGroup sg = getGroup();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < sg.sequences.size(); i++)
        {
            Sequence tmp = (Sequence) sg.sequences.get(i);
            sb.append(tmp.getSequence(sg.getStartRes(), sg.getEndRes() + 1));
            sb.append("\n");
        }

        sb.append("Something amazing will happen soon");
        cap.setText(sb.toString());
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    SequenceGroup getGroup()
    {
        SequenceGroup sg = ap.av.getSelectionGroup();
      // this method won't add a new group if it already exists
        if(sg!=null)
          ap.av.alignment.addGroup(sg);

        return sg;
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    void sequenceName_actionPerformed(ActionEvent e)
    {
        String id = sequence.getName();
        String s = (String) JOptionPane.showInternalInputDialog(ap,
                "Edit sequence name",
                "Edit sequence name (" + sequence.getName() + ")",
                JOptionPane.PLAIN_MESSAGE, null, null, id);

        if (s != null)
        {
            if (s.indexOf(" ") > -1)
            {
                JOptionPane.showMessageDialog(ap,
                    "Spaces have been converted to \"_\"",
                    "No spaces allowed in Sequence Name",
                    JOptionPane.WARNING_MESSAGE);
            }

            s = s.replace(' ', '_');
            sequence.setName(s);
            ap.repaint();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    void unGroupMenuItem_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = ap.av.getSelectionGroup();
        ap.av.alignment.deleteGroup(sg);
        ap.av.setSelectionGroup(null);
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    void pdbMenuItem_actionPerformed(ActionEvent e)
    {
        if (sequence.getPDBId() == null)
        {
            return;
        }

        try
        {
            EBIFetchClient ebi = new EBIFetchClient();
            String[] result = ebi.fetchData("pdb:" + sequence.getPDBId(), null,
                    null);

            PDBfile pdb = new PDBfile(result);

            rotCanvas rc = new rotCanvas(pdb, sequence, ap.av);
            JInternalFrame frame = new JInternalFrame();
            frame.setContentPane(rc);
            Desktop.addInternalFrame(frame,
                sequence.getName() + " " + sequence.getPDBId(), 400, 400);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void outline_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        Color col = JColorChooser.showDialog(this, "Select Outline Colour",
                Color.BLUE);

        if (col != null)
        {
            sg.setOutlineColour(col);
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showBoxes_actionPerformed(ActionEvent e)
    {
        getGroup().setDisplayBoxes(showBoxes.isSelected());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showText_actionPerformed(ActionEvent e)
    {
        getGroup().setDisplayText(showText.isSelected());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showColourText_actionPerformed(ActionEvent e)
    {
        getGroup().setColourText(showColourText.isSelected());
        refresh();
    }

    public void showLink(String url)
    {
      try
      {
        jalview.util.BrowserLauncher.openURL(url);
      }
      catch (Exception ex)
      {
        JOptionPane.showInternalMessageDialog(Desktop.desktop,
            "Unixers: Couldn't find default web browser."
           +"\nAdd the full path to your browser in Preferences.",
           "Web browser not found", JOptionPane.WARNING_MESSAGE );


        ex.printStackTrace();
      }
    }
}
