/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.16 $
 */
public class ScalePanel extends JPanel
{
    protected int offy = 4;

    /** DOCUMENT ME!! */
    public int width;
    protected AlignViewport av;
    AlignmentPanel ap;
    boolean stretchingGroup = false;
    int min; //used by mouseDragged to see if user
    int max; //used by mouseDragged to see if user

    // wants to delete columns
    public ScalePanel(AlignViewport av, AlignmentPanel ap)
    {
        this.av = av;
        this.ap = ap;

        addMouseListener(new MouseAdapter()
            {
                public void mousePressed(MouseEvent evt)
                {
                    doMousePressed(evt);
                }

                public void mouseReleased(MouseEvent evt)
                {
                    doMouseReleased(evt);
                }
            });
        addMouseMotionListener(new MouseMotionAdapter()
            {
                public void mouseDragged(MouseEvent evt)
                {
                    doMouseDragged(evt);
                }
            });
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void doMousePressed(MouseEvent evt)
    {
        int x = evt.getX();
        int res = (x / av.getCharWidth()) + av.getStartRes();
        SequenceGroup sg = null;

        min = res;
        max = res;

        if (av.getColumnSelection().contains(res))
        {
            av.getColumnSelection().removeElement(res);
            ap.annotationPanel.removeEditableColumn(res);
        }
        else
        {
            av.getColumnSelection().addElement(res);
            sg = new SequenceGroup();

            for (int i = 0; i < av.alignment.getSequences().size(); i++)
            {
                sg.addSequence(av.alignment.getSequenceAt(i), false);
            }

            sg.setStartRes(res);
            sg.setEndRes(res);
        }

        av.setSelectionGroup(sg);
        ap.repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void doMouseReleased(MouseEvent evt)
    {
        int x = evt.getX();
        int res = (x / av.getCharWidth()) + av.getStartRes();

        if (av.getColumnSelection() != null)
        {
            for (int i = 0; i < av.getColumnSelection().size(); i++)
            {
                ap.annotationPanel.addEditableColumn(av.getColumnSelection()
                                                       .columnAt(i));
            }
        }

        if (!stretchingGroup)
        {
            ap.repaint();

            return;
        }

        SequenceGroup sg = av.getSelectionGroup();

        if (res > sg.getStartRes())
        {
            sg.setEndRes(res);
        }
        else if (res < sg.getStartRes())
        {
            sg.setStartRes(res);
        }

        stretchingGroup = false;
        ap.repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void doMouseDragged(MouseEvent evt)
    {
        int x = evt.getX();
        int res = (x / av.getCharWidth()) + av.getStartRes();

        if (res < min)
        {
            min = res;
        }

        if (res > max)
        {
            max = res;
        }

        SequenceGroup sg = av.getSelectionGroup();

        if (sg != null)
        {
            stretchingGroup = true;

            if (!av.getColumnSelection().contains(res))
            {
                av.getColumnSelection().addElement(res);
            }

            if (res > sg.getStartRes())
            {
                sg.setEndRes(res);
            }
            else if (res < sg.getStartRes())
            {
                sg.setStartRes(res);
            }

            for (int i = min; i <= max; i++)
            {
                if ((i < sg.getStartRes()) || (i > sg.getEndRes()))
                {
                    av.getColumnSelection().removeElement(i);
                    ap.annotationPanel.removeEditableColumn(res);
                }
                else
                {
                    av.getColumnSelection().addElement(i);
                }
            }

            ap.repaint();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     */
    public void paintComponent(Graphics g)
    {
        drawScale(g, av.getStartRes(), av.getEndRes(), getWidth(), getHeight());
    }

    // scalewidth will normally be screenwidth,
    public void drawScale(Graphics g, int startx, int endx, int width,
        int height)
    {
        Graphics2D gg = (Graphics2D) g;
        gg.setFont(av.getFont());
        gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);

        //Fill in the background
        gg.setColor(Color.white);
        gg.fillRect(0, 0, width, height);
        gg.setColor(Color.black);

        //Fill the selected columns
        ColumnSelection cs = av.getColumnSelection();
        gg.setColor(new Color(220, 0, 0));

        for (int i = 0; i < cs.size(); i++)
        {
            int sel = cs.columnAt(i);

            if ((sel >= startx) && (sel <= endx))
            {
                gg.fillRect((sel - startx) * av.charWidth, 0, av.charWidth,
                    getHeight());
            }
        }

        // Draw the scale numbers
        gg.setColor(Color.black);

        int scalestartx = (startx / 10) * 10;

        FontMetrics fm = gg.getFontMetrics(av.getFont());
        int y = av.charHeight - fm.getDescent();

        if ((scalestartx % 10) == 0)
        {
            scalestartx += 5;
        }

        for (int i = scalestartx; i < endx; i += 5)
        {
            if ((i % 10) == 0)
            {
                gg.drawString(String.valueOf(i),
                    (i - startx - 1) * av.charWidth, y);
                gg.drawLine((int) (((i - startx - 1) * av.charWidth) +
                    (av.charWidth / 2)), y + 2,
                    (int) (((i - startx - 1) * av.charWidth) +
                    (av.charWidth / 2)), y + (fm.getDescent() * 2));
            }
            else
            {
                gg.drawLine((int) (((i - startx - 1) * av.charWidth) +
                    (av.charWidth / 2)), y + fm.getDescent(),
                    (int) (((i - startx - 1) * av.charWidth) +
                    (av.charWidth / 2)), y + (fm.getDescent() * 2));
            }
        }
    }
}
