/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import jalview.schemes.*;

import java.awt.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.22 $
 */
public class SequenceRenderer
{
    AlignViewport av;
    FontMetrics fm;
    boolean renderGaps = true;
    SequenceGroup currentSequenceGroup = null;
    SequenceGroup[] allGroups = null;
    Color resBoxColour;
    Graphics graphics;

    /**
     * Creates a new SequenceRenderer object.
     *
     * @param av DOCUMENT ME!
     */
    public SequenceRenderer(AlignViewport av)
    {
        this.av = av;
    }

    /**
     * DOCUMENT ME!
     *
     * @param b DOCUMENT ME!
     */
    public void renderGaps(boolean b)
    {
        renderGaps = b;
    }

    /**
     * DOCUMENT ME!
     *
     * @param cs DOCUMENT ME!
     * @param seq DOCUMENT ME!
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Color getResidueBoxColour(ColourSchemeI cs, SequenceI seq, int i)
    {
        getBoxColour(cs, seq, i);

        return resBoxColour;
    }

    /**
     * DOCUMENT ME!
     *
     * @param cs DOCUMENT ME!
     * @param seq DOCUMENT ME!
     * @param i DOCUMENT ME!
     */
    void getBoxColour(ColourSchemeI cs, SequenceI seq, int i)
    {
        if (cs != null)
        {
            resBoxColour = cs.findColour(seq.getSequence(i, i + 1), i);
        }
        else
        {
            resBoxColour = Color.white;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param seq DOCUMENT ME!
     * @param sg DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawSequence(Graphics g, SequenceI seq, SequenceGroup[] sg,
        int start, int end, int x1, int y1, int width, int height)
    {
        allGroups = sg;

        graphics = g;

        drawBoxes(seq, start, end, x1, y1, (int) width, height);

        fm = g.getFontMetrics();
        drawText(seq, start, end, x1, y1, (int) width, height);
    }

    /**
     * DOCUMENT ME!
     *
     * @param seq DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawBoxes(SequenceI seq, int start, int end, int x1, int y1,
        int width, int height)
    {
        int i = start;
        int length = seq.getLength();

        int curStart = -1;
        int curWidth = width;

        Color tempColour = null;

        while ((i <= end) && (i < length))
        {
            if (inCurrentSequenceGroup(i))
            {
                if (currentSequenceGroup.getDisplayBoxes())
                {
                    getBoxColour(currentSequenceGroup.cs, seq, i);
                }
                else
                {
                    resBoxColour = Color.white;
                }
            }
            else if (av.getShowBoxes())
            {
                getBoxColour(av.getGlobalColourScheme(), seq, i);
            }
            else
            {
                resBoxColour = Color.white;
            }

            if (resBoxColour != tempColour)
            {
                if (tempColour != null)
                {
                    graphics.fillRect(x1 + (width * (curStart - start)), y1,
                        curWidth, height);
                }

                graphics.setColor(resBoxColour);

                curStart = i;
                curWidth = width;
                tempColour = resBoxColour;
            }
            else
            {
                curWidth += width;
            }

            i++;
        }

        graphics.fillRect(x1 + (width * (curStart - start)), y1, curWidth,
            height);
    }

    /**
     * DOCUMENT ME!
     *
     * @param seq DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawText(SequenceI seq, int start, int end, int x1, int y1,
        int width, int height)
    {
        int pady = height / 5;
        int charOffset = 0;
        char s;

        // Need to find the sequence position here.
        String sequence = seq.getSequence();

        for (int i = start; i <= end; i++)
        {
            graphics.setColor(Color.black);

            if (i < sequence.length())
            {
                s = sequence.charAt(i);
            }
            else
            {
                s = ' ';
            }

            if (!renderGaps && jalview.util.Comparison.isGap(s))
            {
                continue;
            }

            if (inCurrentSequenceGroup(i))
            {
                if (!currentSequenceGroup.getDisplayText())
                {
                    continue;
                }

                if (currentSequenceGroup.getColourText())
                {
                    getBoxColour(currentSequenceGroup.cs, seq, i);
                    graphics.setColor(resBoxColour.darker());
                }
            }
            else
            {
                if (!av.getShowText())
                {
                    continue;
                }

                if (av.getColourText())
                {
                    getBoxColour(av.getGlobalColourScheme(), seq, i);

                    if (av.getShowBoxes())
                    {
                        graphics.setColor(resBoxColour.darker());
                    }
                    else
                    {
                        graphics.setColor(resBoxColour);
                    }
                }
            }

            charOffset = (width - fm.charWidth(s)) / 2;
            graphics.drawString(String.valueOf(s),
                charOffset + x1 + (width * (i - start)), (y1 + height) - pady);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param res DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    boolean inCurrentSequenceGroup(int res)
    {
        if (allGroups == null)
        {
            return false;
        }

        for (int i = 0; i < allGroups.length; i++)
        {
            if ((allGroups[i].getStartRes() <= res) &&
                    (allGroups[i].getEndRes() >= res))
            {
                currentSequenceGroup = allGroups[i];

                return true;
            }
        }

        return false;
    }

    /**
     * DOCUMENT ME!
     *
     * @param seq DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawHighlightedText(SequenceI seq, int start, int end, int x1,
        int y1, int width, int height)
    {
        int pady = height / 5;
        int charOffset = 0;
        graphics.setColor(Color.BLACK);
        graphics.fillRect(x1, y1, width * (end - start + 1), height);
        graphics.setColor(Color.white);

        char s = '~';

        // Need to find the sequence position here.
        for (int i = start; i <= end; i++)
        {
            if (i < seq.getLength())
            {
                s = seq.getSequence().charAt(i);
            }

            charOffset = (width - fm.charWidth(s)) / 2;
            graphics.drawString(String.valueOf(s),
                charOffset + x1 + (width * (i - start)), (y1 + height) - pady);
        }
    }
}
