/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.analysis.*;

import jalview.datamodel.*;

import jalview.io.*;

import jalview.jbgui.*;

import org.jibble.epsgraphics.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import java.io.*;

import java.util.*;

import javax.imageio.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.19 $
 */
public class TreePanel extends GTreePanel
{
    SequenceI[] seq;
    String type;
    String pwtype;
    int start;
    int end;
    TreeCanvas treeCanvas;
    NJTree tree;

    /**
     * Creates a new TreePanel object.
     *
     * @param av DOCUMENT ME!
     * @param seqVector DOCUMENT ME!
     * @param type DOCUMENT ME!
     * @param pwtype DOCUMENT ME!
     * @param s DOCUMENT ME!
     * @param e DOCUMENT ME!
     */
    public TreePanel(AlignViewport av, Vector seqVector, String type,
        String pwtype, int s, int e)
    {
        super();

        this.type = type;
        this.pwtype = pwtype;

        start = s;
        end = e;

        String longestName = "";
        seq = new Sequence[seqVector.size()];

        for (int i = 0; i < seqVector.size(); i++)
        {
            seq[i] = (Sequence) seqVector.elementAt(i);

            if (seq[i].getName().length() > longestName.length())
            {
                longestName = seq[i].getName();
            }
        }

        tree = new NJTree(seq, type, pwtype, start, end);

        treeCanvas = new TreeCanvas(av, tree, scrollPane, longestName);
        treeCanvas.setShowDistances(true); // We know this tree has distances. JBPNote TODO: prolly should add this as a userdefined default
        setViewStateFromTreeCanvas(treeCanvas);
        tree.reCount(tree.getTopNode());
        tree.findHeight(tree.getTopNode());
        scrollPane.setViewportView(treeCanvas);
    }

    /**
     * Creates a new TreePanel object.
     *
     * @param av DOCUMENT ME!
     * @param seqVector DOCUMENT ME!
     * @param newtree DOCUMENT ME!
     * @param type DOCUMENT ME!
     * @param pwtype DOCUMENT ME!
     */
    public TreePanel(AlignViewport av, Vector seqVector, NewickFile newtree,
        String type, String pwtype)
    {
        super();

        // These are probably only  arbitrary strings reflecting source of tree
        this.type = type;
        this.pwtype = pwtype;

        start = 0;
        end = seqVector.size();

        String longestName = "";
        seq = new Sequence[seqVector.size()];

        for (int i = 0; i < seqVector.size(); i++)
        {
            seq[i] = (Sequence) seqVector.elementAt(i);
        }

        // This constructor matches sequence names to treenodes and sets up the tree layouts.
        tree = new NJTree(seq, newtree);

        // Now have to calculate longest name based on the leaves
        Vector leaves = tree.findLeaves(tree.getTopNode(), new Vector());
        boolean has_placeholders = false;

        for (int i = 0; i < leaves.size(); i++)
        {
            SequenceNode lf = (SequenceNode) leaves.elementAt(i);

            if (lf.isPlaceholder())
            {
                has_placeholders = true;
            }

            if (longestName.length() < ((Sequence) lf.element()).getName()
                                            .length())
            {
                longestName = TreeCanvas.PLACEHOLDER +
                    ((Sequence) lf.element()).getName();
            }
        }

        tree.reCount(tree.getTopNode());
        tree.findHeight(tree.getTopNode());

        treeCanvas = new TreeCanvas(av, tree, scrollPane, longestName);
        treeCanvas.setShowBootstrap(newtree.HasBootstrap());
        treeCanvas.setShowDistances(newtree.HasDistances());
        scrollPane.setViewportView(treeCanvas);
        treeCanvas.setMarkPlaceholders(has_placeholders);
        setViewStateFromTreeCanvas(treeCanvas);

        // JBPNote TODO: preference for always marking placeholders in new associated tree
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public NJTree getTree()
    {
        return tree;
    }

    /**
     * DOCUMENT ME!
     *
     * @param tree DOCUMENT ME!
     */
    private void setViewStateFromTreeCanvas(TreeCanvas tree)
    {
        // update view menu state from treeCanvas render state
        distanceMenu.setSelected(tree.showDistances);
        bootstrapMenu.setSelected(tree.showBootstrap);
        placeholdersMenu.setSelected(tree.markPlaceholders);
    }

    /**
     * DOCUMENT ME!
     *
     * @param tree DOCUMENT ME!
     */
    private void setTreeCanvasFromViewState(TreeCanvas tree)
    {
        // update view menu state from treeCanvas render state
        tree.showDistances = distanceMenu.isSelected();
        tree.showBootstrap = bootstrapMenu.isSelected();
        tree.markPlaceholders = placeholdersMenu.isSelected();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void textbox_actionPerformed(ActionEvent e)
    {
        CutAndPasteTransfer cap = new CutAndPasteTransfer();

        StringBuffer buffer = new StringBuffer();

        if (type.equals("AV"))
        {
            buffer.append("Average distance tree using ");
        }
        else
        {
            buffer.append("Neighbour joining tree using ");
        }

        if (pwtype.equals("BL"))
        {
            buffer.append("BLOSUM62");
        }
        else
        {
            buffer.append("PID");
        }

        Desktop.addInternalFrame(cap, buffer.toString(), 500, 100);

        jalview.io.NewickFile fout = new jalview.io.NewickFile(tree.getTopNode());
        cap.setText(fout.print(false, true));
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void saveAsNewick_actionPerformed(ActionEvent e)
    {
        JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty(
                    "LAST_DIRECTORY"));
        chooser.setFileView(new JalviewFileView());
        chooser.setDialogTitle("Save tree as newick file");
        chooser.setToolTipText("Save");

        int value = chooser.showSaveDialog(null);

        if (value == JalviewFileChooser.APPROVE_OPTION)
        {
            String choice = chooser.getSelectedFile().getPath();
            jalview.bin.Cache.setProperty("LAST_DIRECTORY",
                chooser.getSelectedFile().getParent());

            try
            {
                jalview.io.NewickFile fout = new jalview.io.NewickFile(tree.getTopNode());
                String output = fout.print(false, true); // distances only
                java.io.PrintWriter out = new java.io.PrintWriter(new java.io.FileWriter(
                            choice));
                out.println(output);
                out.close();
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void printMenu_actionPerformed(ActionEvent e)
    {
        //Putting in a thread avoids Swing painting problems
        treeCanvas.startPrinting();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void fitToWindow_actionPerformed(ActionEvent e)
    {
        treeCanvas.fitToWindow = fitToWindow.isSelected();
        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void fontSize_actionPerformed(ActionEvent e)
    {
        if (treeCanvas == null)
        {
            return;
        }

        String size = fontSize.getText().substring(fontSize.getText().indexOf("-") +
                1);

        Object selection = JOptionPane.showInternalInputDialog(Desktop.desktop,
                "Select font size", "Font size", JOptionPane.QUESTION_MESSAGE,
                null,
                new String[]
                {
                    "1", "2", "4", "6", "8", "10", "12", "14", "16", "18", "20"
                }, "Font Size - " + size);

        if (selection != null)
        {
            fontSize.setText("Font Size - " + selection);

            int i = Integer.parseInt(selection.toString());
            treeCanvas.setFontSize(i);
        }

        scrollPane.setViewportView(treeCanvas);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void distanceMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setShowDistances(distanceMenu.isSelected());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void bootstrapMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setShowBootstrap(bootstrapMenu.isSelected());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void placeholdersMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setMarkPlaceholders(placeholdersMenu.isSelected());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void epsTree_actionPerformed(ActionEvent e)
    {
        int width = treeCanvas.getWidth();
        int height = treeCanvas.getHeight();

        try
        {
            jalview.io.JalviewFileChooser chooser = new jalview.io.JalviewFileChooser(jalview.bin.Cache.getProperty(
                        "LAST_DIRECTORY"), new String[] { "eps" },
                    new String[] { "Encapsulated Postscript" },
                    "Encapsulated Postscript");
            chooser.setFileView(new jalview.io.JalviewFileView());
            chooser.setDialogTitle("Create EPS file from tree");
            chooser.setToolTipText("Save");

            int value = chooser.showSaveDialog(this);

            if (value != jalview.io.JalviewFileChooser.APPROVE_OPTION)
            {
                return;
            }

            jalview.bin.Cache.setProperty("LAST_DIRECTORY",
                chooser.getSelectedFile().getParent());

            FileOutputStream out = new FileOutputStream(chooser.getSelectedFile());
            EpsGraphics2D pg = new EpsGraphics2D("Tree", out, 0, 0, width,
                    height);

            treeCanvas.draw(pg, width, height);

            pg.flush();
            pg.close();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void pngTree_actionPerformed(ActionEvent e)
    {
        int width = treeCanvas.getWidth();
        int height = treeCanvas.getHeight();

        try
        {
            jalview.io.JalviewFileChooser chooser = new jalview.io.JalviewFileChooser(jalview.bin.Cache.getProperty(
                        "LAST_DIRECTORY"), new String[] { "png" },
                    new String[] { "Portable network graphics" },
                    "Portable network graphics");

            chooser.setFileView(new jalview.io.JalviewFileView());
            chooser.setDialogTitle("Create PNG image from tree");
            chooser.setToolTipText("Save");

            int value = chooser.showSaveDialog(this);

            if (value != jalview.io.JalviewFileChooser.APPROVE_OPTION)
            {
                return;
            }

            jalview.bin.Cache.setProperty("LAST_DIRECTORY",
                chooser.getSelectedFile().getParent());

            FileOutputStream out = new FileOutputStream(chooser.getSelectedFile());

            BufferedImage bi = new BufferedImage(width, height,
                    BufferedImage.TYPE_INT_RGB);
            Graphics png = bi.getGraphics();

            treeCanvas.draw(png, width, height);

            ImageIO.write(bi, "png", out);
            out.close();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }
}
