/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.io;

import jalview.datamodel.*;

import java.io.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.11 $
 */
public class BLCFile extends AlignFile
{
    Vector titles;

    /**
     * Creates a new BLCFile object.
     */
    public BLCFile()
    {
    }

    /**
     * Creates a new BLCFile object.
     *
     * @param inStr DOCUMENT ME!
     */
    public BLCFile(String inStr)
    {
        super(inStr);
    }

    /**
     * Creates a new BLCFile object.
     *
     * @param inFile DOCUMENT ME!
     * @param type DOCUMENT ME!
     *
     * @throws IOException DOCUMENT ME!
     */
    public BLCFile(String inFile, String type) throws IOException
    {
        super(inFile, type);
    }

    /**
     * DOCUMENT ME!
     */
    public void initData()
    {
        super.initData();
        titles = new Vector();
    }

    /**
     * DOCUMENT ME!
     */
    public void parse()
    {
        boolean idsFound = false;
        Vector ids = new Vector();
        StringBuffer[] seqstrings;
        Vector starts = new Vector();
        Vector ends = new Vector();

        String line = null;

        try
        {
            do
            {
                line = nextLine();

                // seek end of ids
                if (line.indexOf("*") > -1)
                {
                    idsFound = true;

                    break;
                }

                int abracket = line.indexOf(">");

                if (abracket > -1)
                {
                    if (line.indexOf(" ") > -1) //
                    {
                        ///Colur it be this format?
                        //>54402046         0             1   137   137:
                        // or this??
                        //     1   >L1H14       30539  343
                        try
                        {
                            ids.addElement(line.substring(abracket + 1,
                                    line.indexOf(" ", abracket + 1)));

                            // remove p Value
                            line = line.substring(abracket + 1);
                            line = line.substring(line.indexOf(" ") + 1);
                            line = line.trim();
                            line = line.substring(line.indexOf(" ") + 1);
                            line = line.trim();

                            int value = Integer.parseInt(line.substring(0,
                                        line.indexOf(" ")));
                            starts.addElement(value + "");
                            line = line.substring(line.indexOf(" ") + 1);
                            line = line.trim();
                            value = Integer.parseInt(line.substring(0,
                                        line.indexOf(" ")));
                            ends.addElement(value + "");
                        }
                        catch (Exception ex)
                        {
                            System.err.println("Error during blockfile read.");
                            ex.printStackTrace();
                            starts.addElement("1");
                            ends.addElement("-1");
                        }
                    }
                    else
                    {
                        if (line.indexOf("/") > -1)
                        {
                            ids.addElement(line.substring(abracket + 1,
                                    line.indexOf("/")));
                            line = line.substring(line.indexOf("/") + 1);
                            starts.addElement(line.substring(0,
                                    line.indexOf("-")));
                            ends.addElement(line.substring(line.indexOf("-") +
                                    1));
                        }
                        else
                        {
                            ids.addElement(line.substring(abracket + 1));
                            starts.addElement("1");
                            ends.addElement("-1");
                        }
                    }
                }
            }
            while (!idsFound);

            int starCol = line.indexOf("*");
            seqstrings = new StringBuffer[ids.size()];

            for (int i = 0; i < ids.size(); i++)
            {
                if (seqstrings[i] == null)
                {
                    seqstrings[i] = new StringBuffer();
                }
            }

            while ((line = nextLine()).indexOf("*") == -1)
            {
                for (int i = 0; i < ids.size(); i++)
                {
                    if (line.length() > (i + starCol))
                    {
                        seqstrings[i].append(line.charAt(i + starCol));
                    }
                }
            }

            for (int i = 0; i < ids.size(); i++)
            {
                Sequence newSeq = new Sequence(ids.elementAt(i).toString(),
                        seqstrings[i].toString(),
                        Integer.parseInt(starts.elementAt(i).toString()),
                        Integer.parseInt(ends.elementAt(i).toString()));
                seqs.addElement(newSeq);
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String print()
    {
        return print(getSeqsAsArray());
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public static String print(SequenceI[] s)
    {
        StringBuffer out = new StringBuffer();

        int i = 0;
        int max = -1;

        while ((i < s.length) && (s[i] != null))
        {
            out.append(">" + s[i].getName() + "/" + s[i].getStart() + "-" +
                s[i].getEnd() + "\n");

            if (s[i].getSequence().length() > max)
            {
                max = s[i].getSequence().length();
            }

            i++;
        }

        out.append("* iteration 1\n");

        for (int j = 0; j < max; j++)
        {
            i = 0;

            while ((i < s.length) && (s[i] != null))
            {
                if (s[i].getSequence().length() > j)
                {
                    out.append(s[i].getSequence().substring(j, j + 1));
                }
                else
                {
                    out.append("-");
                }

                i++;
            }

            out.append("\n");
        }

        out.append("*\n");

        return out.toString();
    }
}
