/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.io;

import java.io.*;
import java.util.*;

import jalview.analysis.*;
import jalview.datamodel.*;

public class PIRFile
    extends AlignFile
{
  Vector words = new Vector(); //Stores the words in a line after splitting

  public PIRFile()
  {
  }

  public PIRFile(String inStr)
  {
    super(inStr);
  }

  public PIRFile(String inFile, String type)
      throws IOException
  {
    super(inFile, type);
  }

  public void parse()
  {
    try
    {
      String id;
      String start;
      String end;
      StringBuffer sequence;
      String line = null;

      while ( (line = nextLine()) != null)
      {
        if(line.length()==0)
        {
          //System.out.println("blank line");
          continue;
        }

        id = "No id";
        start = "1";
        end = "-1";

        try
        {
          int slashIndex = line.indexOf("/");
          if(slashIndex!=-1)
          {
            id = line.substring(line.indexOf(";") + 1, line.indexOf("/"));
            line = line.substring(line.indexOf("/") + 1);
            start = line.substring(0, line.indexOf("-"));
            end = line.substring(line.indexOf("-") + 1);
          }
          else
          {
            id = line.substring(line.indexOf(";")+1);
          }
        }
        catch (Exception ex)
        {    }// Default id, start and end unchanged

        sequence = new StringBuffer();

        line = nextLine(); // this is the title line

        boolean starFound = false;

        while(!starFound)
        {
          line = nextLine();
          sequence.append(line);

          if (line == null)
            break;

          if (line.indexOf("*") > -1)
          {
            starFound = true;
          }
        }

        if(sequence.length()>0)
        {
          sequence.setLength(sequence.length() - 1);

          Sequence newSeq = new Sequence(id, sequence.toString(),
                                         Integer.parseInt(start),
                                         Integer.parseInt(end));
          seqs.addElement(newSeq);
        }
      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }

  public String print()
  {
    return print(getSeqsAsArray());
  }

  public static String print(SequenceI[] s)
  {
    return print(s, 72, true);
  }

  public static String print(SequenceI[] s, int len)
  {
    return print(s, len, true);
  }

  public static String print(SequenceI[] s, int len, boolean gaps)
  {
    StringBuffer out = new StringBuffer();
    int i = 0;

    while ( (i < s.length) && (s[i] != null))
    {
      String seq = "";

      if (gaps)
      {
        seq = s[i].getSequence() + "*";
      }
      else
      {
        seq = AlignSeq.extractGaps(s[i].getSequence(), "-");
        seq = AlignSeq.extractGaps(seq, ".");
        seq = AlignSeq.extractGaps(seq, " ");
        seq = seq + "*";
      }

      out.append(">P1;" + s[i].getName() + "/" + s[i].getStart() + "-" +
                 s[i].getEnd() + "\n");
      out.append(" Dummy title\n");

      int nochunks = (seq.length() / len) + 1;

      for (int j = 0; j < nochunks; j++)
      {
        int start = j * len;
        int end = start + len;

        if (end < seq.length())
        {
          out.append(seq.substring(start, end) + "\n");
        }
        else if (start < seq.length())
        {
          out.append(seq.substring(start) + "\n");
        }
      }

      i++;
    }

    return out.toString();
  }

  public static void main(String[] args)
  {
    String inStr = ">P1;LCAT_MOUSE_90.35\nMGLPGSPWQRVLLLLGLLLPPATPFWLLNVLFPPHTTPKAELSNHTRPVILVPGCLGNRLEAKLDKPDVVNW\nMCYRKTEDFFTIWLDFNLFLPLGVDCWIDNTRIVYNHSSGRVSNAPGVQIRVPGFGKTESVEYVDDNKLAGY\n\n>LCAT_PAPAN_95.78\nMGPPGSPWQWVPLLLGLLLPPAAPFWLLNVLFPPHTTPKAELSNHTRPVILVPGCLGNQLEAKLDKPDVVNW\nMCYRKTEDFFTIWLDLNMFLPLGVDCWIDNTRVVYNRSSGLVSNAPGVQIRVPGFGKTYSVEYLDSSKLAGY\nLHTLVQNLVNNGYVRDETVRAAPYDWRLEPGQQEEYYHKLAGLVEEMHAAYGKPVFLIGHSLGCLHLLYFLL\n";
    PIRFile fa = new PIRFile(inStr);
  }
}
