/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.jbappletgui;

import java.awt.*;
import java.awt.event.*;


public class GAlignFrame extends Frame {
    protected MenuBar alignFrameMenuBar = new MenuBar();
    protected Menu fileMenu = new Menu();
    protected MenuItem closeMenuItem = new MenuItem();
    protected Menu editMenu = new Menu();
    protected Menu viewMenu = new Menu();
    protected Menu colourMenu = new Menu();
    protected Menu calculateMenu = new Menu();
    protected MenuItem selectAllSequenceMenuItem = new MenuItem();
    protected MenuItem deselectAllSequenceMenuItem = new MenuItem();
    protected MenuItem invertSequenceMenuItem = new MenuItem();
    protected MenuItem remove2LeftMenuItem = new MenuItem();
    protected MenuItem remove2RightMenuItem = new MenuItem();
    protected MenuItem removeGappedColumnMenuItem = new MenuItem();
    protected MenuItem removeAllGapsMenuItem = new MenuItem();
    protected CheckboxMenuItem viewBoxesMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem viewTextMenuItem = new CheckboxMenuItem();
    protected MenuItem sortPairwiseMenuItem = new MenuItem();
    protected MenuItem sortIDMenuItem = new MenuItem();
    protected MenuItem sortGroupMenuItem = new MenuItem();
    protected MenuItem removeRedundancyMenuItem = new MenuItem();
    protected MenuItem pairwiseAlignmentMenuItem = new MenuItem();
    protected MenuItem PCAMenuItem = new MenuItem();
    protected MenuItem averageDistanceTreeMenuItem = new MenuItem();
    protected MenuItem neighbourTreeMenuItem = new MenuItem();
    BorderLayout borderLayout1 = new BorderLayout();
    public Label statusBar = new Label();
    protected Menu outputTextboxMenu = new Menu();
    protected MenuItem clustalColour = new MenuItem();
    protected MenuItem zappoColour = new MenuItem();
    protected MenuItem taylorColour = new MenuItem();
    protected MenuItem hydrophobicityColour = new MenuItem();
    protected MenuItem helixColour = new MenuItem();
    protected MenuItem strandColour = new MenuItem();
    protected MenuItem turnColour = new MenuItem();
    protected MenuItem buriedColour = new MenuItem();
    protected MenuItem userDefinedColour = new MenuItem();
    protected MenuItem PIDColour = new MenuItem();
    protected MenuItem BLOSUM62Colour = new MenuItem();
    MenuItem njTreeBlosumMenuItem = new MenuItem();
    MenuItem avDistanceTreeBlosumMenuItem = new MenuItem();
    protected CheckboxMenuItem annotationPanelMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem colourTextMenuItem = new CheckboxMenuItem();
    MenuItem overviewMenuItem = new MenuItem();
    protected MenuItem undoMenuItem = new MenuItem();
    protected MenuItem redoMenuItem = new MenuItem();
    protected CheckboxMenuItem conservationMenuItem = new CheckboxMenuItem();
    MenuItem noColourmenuItem = new MenuItem();
    protected CheckboxMenuItem wrapMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem renderGapsMenuItem = new CheckboxMenuItem();
    MenuItem findMenuItem = new MenuItem();
    Menu searchMenu = new Menu();
    protected CheckboxMenuItem abovePIDThreshold = new CheckboxMenuItem();
    protected MenuItem nucleotideColour = new MenuItem();
    MenuItem deleteGroups = new MenuItem();
    MenuItem delete = new MenuItem();
    MenuItem copy = new MenuItem();
    MenuItem cut = new MenuItem();
    Menu Menu1 = new Menu();
    MenuItem pasteNew = new MenuItem();
    MenuItem pasteThis = new MenuItem();
    protected CheckboxMenuItem applyToAllGroups = new CheckboxMenuItem();
    protected MenuItem font = new MenuItem();
    protected CheckboxMenuItem fullSeqId = new CheckboxMenuItem();
    protected CheckboxMenuItem scaleAbove = new CheckboxMenuItem();
    protected CheckboxMenuItem scaleLeft = new CheckboxMenuItem();
    protected CheckboxMenuItem scaleRight = new CheckboxMenuItem();
    MenuItem modifyPID = new MenuItem();
    MenuItem modifyConservation = new MenuItem();
    protected Menu sortByTreeMenu = new Menu();
    Menu sort = new Menu();
    Menu calculate = new Menu();
    MenuItem inputText = new MenuItem();
    Menu helpMenu = new Menu();
    MenuItem documentation = new MenuItem();
    MenuItem about = new MenuItem();

    public GAlignFrame() {
        try {
            jbInit();
            setMenuBar(alignFrameMenuBar);

            MenuItem item;

            // dynamically fill save as menu with available formats
            for (int i = 0; i < jalview.io.FormatAdapter.formats.size(); i++) {
                item = new MenuItem((String) jalview.io.FormatAdapter.formats.elementAt(
                            i));
                item.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            saveAs_actionPerformed(e);
                        }
                    });

                item = new MenuItem((String) jalview.io.FormatAdapter.formats.elementAt(
                            i));
                item.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            outputText_actionPerformed(e);
                        }
                    });

                outputTextboxMenu.add(item);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        fileMenu.setLabel("File");
        closeMenuItem.setLabel("Close");
        closeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    closeMenuItem_actionPerformed(e);
                }
            });
        editMenu.setLabel("Edit");
        viewMenu.setLabel("View");
        colourMenu.setLabel("Colour");
        calculateMenu.setLabel("Calculate");
        selectAllSequenceMenuItem.setLabel("Select all");
        selectAllSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    selectAllSequenceMenuItem_actionPerformed(e);
                }
            });
        deselectAllSequenceMenuItem.setLabel("Deselect All");
        deselectAllSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    deselectAllSequenceMenuItem_actionPerformed(e);
                }
            });
        invertSequenceMenuItem.setLabel("Invert Selection");
        invertSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    invertSequenceMenuItem_actionPerformed(e);
                }
            });
        remove2LeftMenuItem.setLabel("Remove Left");
        remove2LeftMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    remove2LeftMenuItem_actionPerformed(e);
                }
            });
        remove2RightMenuItem.setLabel("Remove Right");
        remove2RightMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    remove2RightMenuItem_actionPerformed(e);
                }
            });
        removeGappedColumnMenuItem.setLabel("Remove Empty Columns");
        removeGappedColumnMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeGappedColumnMenuItem_actionPerformed(e);
                }
            });
        removeAllGapsMenuItem.setLabel("Remove All Gaps");
        removeAllGapsMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeAllGapsMenuItem_actionPerformed(e);
                }
            });
        viewBoxesMenuItem.setLabel("Boxes");
        viewBoxesMenuItem.setState(true);
        viewBoxesMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    viewBoxesMenuItem_actionPerformed(null);
                }
            });
        viewTextMenuItem.setLabel("Text");
        viewTextMenuItem.setState(true);
        viewTextMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    viewTextMenuItem_actionPerformed(null);
                }
            });
        sortPairwiseMenuItem.setLabel("by Pairwise Identity");
        sortPairwiseMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortPairwiseMenuItem_actionPerformed(e);
                }
            });
        sortIDMenuItem.setLabel("by ID");
        sortIDMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortIDMenuItem_actionPerformed(e);
                }
            });
        sortGroupMenuItem.setLabel("by Group");
        sortGroupMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortGroupMenuItem_actionPerformed(e);
                }
            });
        removeRedundancyMenuItem.setLabel("Remove Redundancy...");
        removeRedundancyMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeRedundancyMenuItem_actionPerformed(e);
                }
            });
        pairwiseAlignmentMenuItem.setLabel("Pairwise Alignments...");
        pairwiseAlignmentMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pairwiseAlignmentMenuItem_actionPerformed(e);
                }
            });
        PCAMenuItem.setLabel("Principal Component Analysis");
        PCAMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    PCAMenuItem_actionPerformed(e);
                }
            });
        averageDistanceTreeMenuItem.setLabel(
            "Average Distance Using % Identity");
        averageDistanceTreeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    averageDistanceTreeMenuItem_actionPerformed(e);
                }
            });
        neighbourTreeMenuItem.setLabel("Neighbour Joining Using % Identity");
        neighbourTreeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    neighbourTreeMenuItem_actionPerformed(e);
                }
            });
        alignFrameMenuBar.setFont(new java.awt.Font("Verdana", 0, 11));
        statusBar.setBackground(Color.white);
        statusBar.setFont(new java.awt.Font("Verdana", 0, 11));
        statusBar.setText("Status bar");
        outputTextboxMenu.setLabel("Output to Textbox");
        clustalColour.setLabel("Clustalx");

        clustalColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    clustalColour_actionPerformed(e);
                }
            });
        zappoColour.setLabel("Zappo");
        zappoColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    zappoColour_actionPerformed(e);
                }
            });
        taylorColour.setLabel("Taylor");
        taylorColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    taylorColour_actionPerformed(e);
                }
            });
        hydrophobicityColour.setLabel("Hydrophobicity");
        hydrophobicityColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    hydrophobicityColour_actionPerformed(e);
                }
            });
        helixColour.setLabel("Helix Propensity");
        helixColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    helixColour_actionPerformed(e);
                }
            });
        strandColour.setLabel("Strand Propensity");
        strandColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    strandColour_actionPerformed(e);
                }
            });
        turnColour.setLabel("Turn Propensity");
        turnColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    turnColour_actionPerformed(e);
                }
            });
        buriedColour.setLabel("Buried Index");
        buriedColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    buriedColour_actionPerformed(e);
                }
            });
        userDefinedColour.setLabel("User Defined...");
        userDefinedColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    userDefinedColour_actionPerformed(e);
                }
            });
        PIDColour.setLabel("Percentage Identity");
        PIDColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    PIDColour_actionPerformed(e);
                }
            });
        BLOSUM62Colour.setLabel("BLOSUM62 Score");
        BLOSUM62Colour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    BLOSUM62Colour_actionPerformed(e);
                }
            });
        avDistanceTreeBlosumMenuItem.setLabel(
            "Average Distance Using BLOSUM62");
        avDistanceTreeBlosumMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    avTreeBlosumMenuItem_actionPerformed(e);
                }
            });
        njTreeBlosumMenuItem.setLabel("Neighbour Joining Using BLOSUM62");
        njTreeBlosumMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    njTreeBlosumMenuItem_actionPerformed(e);
                }
            });
        annotationPanelMenuItem.setActionCommand("");
        annotationPanelMenuItem.setState(true);
        annotationPanelMenuItem.setLabel("Show Annotations");
        annotationPanelMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    annotationPanelMenuItem_actionPerformed(null);
                }
            });
        colourTextMenuItem.setLabel("Colour Text");
        colourTextMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    colourTextMenuItem_actionPerformed(null);
                }
            });
        overviewMenuItem.setLabel("Overview Window");
        overviewMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    overviewMenuItem_actionPerformed(e);
                }
            });
        undoMenuItem.setEnabled(false);
        undoMenuItem.setLabel("Undo");
        undoMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    undoMenuItem_actionPerformed(e);
                }
            });
        redoMenuItem.setEnabled(false);
        redoMenuItem.setLabel("Redo");
        redoMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    redoMenuItem_actionPerformed(e);
                }
            });
        conservationMenuItem.setLabel("by Conservation");
        conservationMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    conservationMenuItem_actionPerformed(null);
                }
            });
        noColourmenuItem.setLabel("None");
        noColourmenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    noColourmenuItem_actionPerformed(e);
                }
            });
        wrapMenuItem.setLabel("Wrap");
        wrapMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    wrapMenuItem_actionPerformed(null);
                }
            });
        renderGapsMenuItem.setLabel("Show Gaps");
        renderGapsMenuItem.setState(true);
        renderGapsMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    renderGapsMenuItem_actionPerformed(null);
                }
            });
        findMenuItem.setLabel("Find...");
        findMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    findMenuItem_actionPerformed(e);
                }
            });
        searchMenu.setLabel("Search");

        abovePIDThreshold.setLabel("Above Identity Threshold");
        abovePIDThreshold.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    abovePIDThreshold_actionPerformed(null);
                }
            });
        nucleotideColour.setLabel("Nucleotide");
        nucleotideColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    nucleotideColour_actionPerformed(e);
                }
            });
        deleteGroups.setLabel("Undefine Groups");
        deleteGroups.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    deleteGroups_actionPerformed(e);
                }
            });
        copy.setLabel("Copy");
        copy.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    copy_actionPerformed(e);
                }
            });
        cut.setLabel("Cut");
        cut.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    cut_actionPerformed(e);
                }
            });
        delete.setLabel("Delete");
        delete.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    delete_actionPerformed(e);
                }
            });
        Menu1.setLabel("Paste");
        pasteNew.setLabel("To New Alignment");
        pasteNew.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pasteNew_actionPerformed(e);
                }
            });
        pasteThis.setLabel("Add To This Alignment");
        pasteThis.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pasteThis_actionPerformed(e);
                }
            });
        applyToAllGroups.setLabel("Apply Colour To All Groups");
        applyToAllGroups.setState(true);
        applyToAllGroups.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    applyToAllGroups_actionPerformed(null);
                }
            });
        font.setLabel("Font...");
        font.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    font_actionPerformed(e);
                }
            });
        fullSeqId.setLabel("Show Full Sequence ID");
        fullSeqId.setState(true);
        fullSeqId.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    fullSeqId_actionPerformed(null);
                }
            });
        scaleAbove.setLabel("Scale Above");
        scaleAbove.setState(true);
        scaleAbove.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleAbove_actionPerformed(null);
                }
            });
        scaleLeft.setEnabled(false);
        scaleLeft.setState(true);
        scaleLeft.setLabel("Scale Left");
        scaleLeft.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleLeft_actionPerformed(null);
                }
            });
        scaleRight.setEnabled(false);
        scaleRight.setState(true);
        scaleRight.setLabel("Scale Right");
        scaleRight.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleRight_actionPerformed(null);
                }
            });
        modifyPID.setLabel("Modify Identity Threshold...");
        modifyPID.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    modifyPID_actionPerformed(e);
                }
            });
        modifyConservation.setLabel("Modify Conservation Threshold...");
        modifyConservation.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    modifyConservation_actionPerformed(e);
                }
            });
        sortByTreeMenu.setLabel("By Tree Order");
        sort.setLabel("Sort");
        calculate.setLabel("Calculate Tree");
        inputText.setLabel("Input from textbox");
        inputText.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    inputText_actionPerformed(e);
                }
            });

        helpMenu.setLabel("Help");
        documentation.setLabel("Documentation");
        documentation.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    documentation_actionPerformed(e);
                }
            });

        about.setLabel("About...");
        about.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    about_actionPerformed(e);
                }
            });

        alignFrameMenuBar.add(fileMenu);
        alignFrameMenuBar.add(editMenu);
        alignFrameMenuBar.add(searchMenu);
        alignFrameMenuBar.add(viewMenu);
        alignFrameMenuBar.add(colourMenu);
        alignFrameMenuBar.add(calculateMenu);
        alignFrameMenuBar.add(helpMenu);
        fileMenu.add(inputText);
        fileMenu.add(outputTextboxMenu);
        fileMenu.addSeparator();
        fileMenu.add(closeMenuItem);
        editMenu.add(undoMenuItem);
        editMenu.add(redoMenuItem);
        editMenu.add(cut);
        editMenu.add(copy);
        editMenu.add(Menu1);
        editMenu.add(delete);
        editMenu.addSeparator();
        editMenu.add(selectAllSequenceMenuItem);
        editMenu.add(deselectAllSequenceMenuItem);
        editMenu.add(invertSequenceMenuItem);
        editMenu.add(deleteGroups);
        editMenu.addSeparator();
        editMenu.add(remove2LeftMenuItem);
        editMenu.add(remove2RightMenuItem);
        editMenu.add(removeGappedColumnMenuItem);
        editMenu.add(removeAllGapsMenuItem);
        editMenu.add(removeRedundancyMenuItem);
        searchMenu.add(findMenuItem);
        viewMenu.add(font);
        viewMenu.addSeparator();
        viewMenu.add(wrapMenuItem);
        viewMenu.add(scaleAbove);
        viewMenu.add(scaleLeft);
        viewMenu.add(scaleRight);
        viewMenu.addSeparator();
        viewMenu.add(fullSeqId);
        viewMenu.add(viewBoxesMenuItem);
        viewMenu.add(viewTextMenuItem);
        viewMenu.add(colourTextMenuItem);
        viewMenu.add(renderGapsMenuItem);
        viewMenu.add(annotationPanelMenuItem);
        viewMenu.addSeparator();
        viewMenu.add(overviewMenuItem);
        colourMenu.add(applyToAllGroups);
        colourMenu.addSeparator();
        colourMenu.add(noColourmenuItem);
        colourMenu.add(clustalColour);
        colourMenu.add(BLOSUM62Colour);
        colourMenu.add(PIDColour);
        colourMenu.add(zappoColour);
        colourMenu.add(taylorColour);
        colourMenu.add(hydrophobicityColour);
        colourMenu.add(helixColour);
        colourMenu.add(strandColour);
        colourMenu.add(turnColour);
        colourMenu.add(buriedColour);
        colourMenu.add(nucleotideColour);
        colourMenu.add(userDefinedColour);
        colourMenu.addSeparator();
        colourMenu.add(conservationMenuItem);
        colourMenu.add(modifyConservation);
        colourMenu.add(abovePIDThreshold);
        colourMenu.add(modifyPID);
        calculateMenu.add(sort);
        calculateMenu.add(calculate);
        calculateMenu.addSeparator();
        calculateMenu.add(pairwiseAlignmentMenuItem);
        calculateMenu.add(PCAMenuItem);
        this.add(statusBar, BorderLayout.SOUTH);
        Menu1.add(pasteNew);
        Menu1.add(pasteThis);
        sort.add(sortIDMenuItem);
        sort.add(sortByTreeMenu);
        sort.add(sortGroupMenuItem);
        sort.add(sortPairwiseMenuItem);
        calculate.add(averageDistanceTreeMenuItem);
        calculate.add(neighbourTreeMenuItem);
        calculate.add(avDistanceTreeBlosumMenuItem);
        calculate.add(njTreeBlosumMenuItem);
        helpMenu.add(documentation);
        helpMenu.add(about);
    }

    protected void saveAs_actionPerformed(ActionEvent e) {
    }

    protected void outputText_actionPerformed(ActionEvent e) {
    }

    protected void htmlMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void closeMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void redoMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void undoMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void selectAllSequenceMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void deselectAllSequenceMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void invertSequenceMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void remove2LeftMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void remove2RightMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void removeGappedColumnMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void removeAllGapsMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void wrapMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void viewBoxesMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void viewTextMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void colourTextMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void annotationPanelMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void overviewMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void sortPairwiseMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void sortIDMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void sortGroupMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void removeRedundancyMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void pairwiseAlignmentMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void PCAMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void averageDistanceTreeMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void neighbourTreeMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void njTreeBlosumMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void avTreeBlosumMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void clustalAlignMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void clustalColour_actionPerformed(ActionEvent e) {
    }

    protected void zappoColour_actionPerformed(ActionEvent e) {
    }

    protected void taylorColour_actionPerformed(ActionEvent e) {
    }

    protected void hydrophobicityColour_actionPerformed(ActionEvent e) {
    }

    protected void helixColour_actionPerformed(ActionEvent e) {
    }

    protected void strandColour_actionPerformed(ActionEvent e) {
    }

    protected void turnColour_actionPerformed(ActionEvent e) {
    }

    protected void buriedColour_actionPerformed(ActionEvent e) {
    }

    protected void userDefinedColour_actionPerformed(ActionEvent e) {
    }

    protected void PIDColour_actionPerformed(ActionEvent e) {
    }

    protected void BLOSUM62Colour_actionPerformed(ActionEvent e) {
    }

    protected void noColourmenuItem_actionPerformed(ActionEvent e) {
    }

    protected void conservationMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void printMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void renderGapsMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void findMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void abovePIDThreshold_actionPerformed(ActionEvent e) {
    }

    public void sequenceFeatures_actionPerformed(ActionEvent actionEvent) {
    }

    protected void nucleotideColour_actionPerformed(ActionEvent e) {
    }

    protected void deleteGroups_actionPerformed(ActionEvent e) {
    }

    protected void copy_actionPerformed(ActionEvent e) {
    }

    protected void cut_actionPerformed(ActionEvent e) {
    }

    protected void delete_actionPerformed(ActionEvent e) {
    }

    protected void pasteNew_actionPerformed(ActionEvent e) {
    }

    protected void pasteThis_actionPerformed(ActionEvent e) {
    }

    protected void applyToAllGroups_actionPerformed(ActionEvent e) {
    }

    protected void createPNG_actionPerformed(ActionEvent e) {
    }

    protected void font_actionPerformed(ActionEvent e) {
    }

    protected void fullSeqId_actionPerformed(ActionEvent e) {
    }

    protected void epsFile_actionPerformed(ActionEvent e) {
    }

    protected void LoadtreeMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void jpred_actionPerformed(ActionEvent e) {
    }

    protected void scaleAbove_actionPerformed(ActionEvent e) {
    }

    protected void scaleLeft_actionPerformed(ActionEvent e) {
    }

    protected void scaleRight_actionPerformed(ActionEvent e) {
    }

    protected void modifyPID_actionPerformed(ActionEvent e) {
    }

    protected void modifyConservation_actionPerformed(ActionEvent e) {
    }

    protected void preferences_actionPerformed(ActionEvent e) {
    }

    protected void msaAlignMenuItem_actionPerformed(ActionEvent e) {
    }

    protected void inputText_actionPerformed(ActionEvent e) {
    }

    protected void documentation_actionPerformed(ActionEvent e) {
    }

    protected void about_actionPerformed(ActionEvent e) {
    }
}
