/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.schemes;

import jalview.analysis.*;

import java.awt.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.15 $
 */
public class ResidueColourScheme implements ColourSchemeI
{

    boolean conservationColouring = false;
    boolean consensusColouring = false;

    Color[] colors;
    int threshold = 0;

    /* Set when threshold colouring to either pid_gaps or pid_nogaps*/
    protected String ignoreGaps = "pid_gaps";

    /** Consenus as a hashtable array */
    Hashtable [] consensus;

    /** Conservation string as a char array */
   char [] conservation;

   /** DOCUMENT ME!! */
   int inc = 30;

   /**
    * The colour to be calculated, manipulated and returned
    */
   Color currentColour = null;



    /**
     * Creates a new ResidueColourScheme object.
     *
     * @param colors DOCUMENT ME!
     * @param threshold DOCUMENT ME!
     */
    public ResidueColourScheme(Color[] colours, int threshold)
    {
        this.colors = colours;
        this.threshold = threshold;
    }

    /**
     * Creates a new ResidueColourScheme object.
     */
    public ResidueColourScheme()
    {
    }

    /**
    * Find a colour without an index in a sequence
    */
   public Color findColour(String aa)
   {
       return colors[((Integer) (ResidueProperties.aaHash.get(aa))).intValue()];
   }



   public Color findColour(String s, int j)
   {

       int index = ((Integer) (ResidueProperties.aaHash.get(s))).intValue();

       if ((threshold == 0) || aboveThreshold(ResidueProperties.aa[index], j))
       {
           currentColour = colors[index];
       }
       else
       {
           currentColour = Color.white;
       }

       if(conservationColouring)
         applyConservation(j);


       return currentColour;
   }


    /**
     * Get the percentage threshold for this colour scheme
     *
     * @return Returns the percentage threshold
     */
    public int getThreshold()
    {
        return threshold;
    }

    /**
     * DOCUMENT ME!
     *
     * @param ct DOCUMENT ME!
     */
    public void setThreshold(int ct, boolean ignoreGaps)
    {
        threshold = ct;
        if(ignoreGaps)
          this.ignoreGaps = "pid_nogaps";
        else
          this.ignoreGaps = "pid_gaps";
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     * @param j DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean aboveThreshold(String s, int j)
    {
        if ((((Integer) consensus[j].get("maxCount")).intValue() != -1) &&
                consensus[j].contains(s))
        {
            if (((Float)consensus[j].get(ignoreGaps)).floatValue() >= threshold)
            {
                return true;
            }
        }

        return false;
    }


    public boolean conservationApplied()
    {
      return conservationColouring;
    }

    public void setConservationInc(int i)
    {
      inc = i;
    }

    public int getConservationInc()
    {
      return inc;
    }

    /**
     * DOCUMENT ME!
     *
     * @param consensus DOCUMENT ME!
     */
    public void setConsensus(Vector vconsensus)
    {
       int i, iSize=vconsensus.size();
       consensus = new Hashtable[iSize];
       for(i=0; i<iSize; i++)
        consensus[i] = (Hashtable)vconsensus.elementAt(i);
    }


    public void setConservation(Conservation cons)
    {
      if(cons==null)
      {
        conservationColouring = false;
        conservation = null;
      }
      else
      {
        conservationColouring = true;
        int i, iSize = cons.getConsSequence().getLength();
        conservation = new char[iSize];
        for (i = 0; i < iSize; i++)
          conservation[i] = cons.getConsSequence().getCharAt(i);
      }

    }


    /**
    * DOCUMENT ME!
    *
    * @param s DOCUMENT ME!
    * @param i DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   void applyConservation(int i)
   {
       if ((conservation[i] != '*') && (conservation[i] != '+'))
       {
           int tmp = 10;
           int t = 0;

           if (!jalview.util.Comparison.isGap(conservation[i]))
           {
               t = conservation[i]-'0';
           }

           while (tmp >= t)
           {
               lighter(inc);
               tmp--;
           }
       }

   }

    /**
    * DOCUMENT ME!
    *
    * @param c DOCUMENT ME!
    * @param inc DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   void lighter(int inc)
   {
       int red = currentColour.getRed();
       int blue = currentColour.getBlue();
       int green = currentColour.getGreen();

       if (red < (255 - inc))
       {
           red = red + inc;
       }
       else
       {
           red = 255;
       }

       if (blue < (255 - inc))
       {
           blue = blue + inc;
       }
       else
       {
           blue = 255;
       }

       if (green < (255 - inc))
       {
           green = green + inc;
       }
       else
       {
           green = 255;
       }

       currentColour = new Color(red, green, blue);
   }


}
